Package: scPipe
Title: Pipeline for single cell multi-omic data pre-processing
Date: 2022-10-12
Version: 2.12.0
Type: Package
Authors@R: c(person("Luyi", "Tian", role=c("aut"), email="tian.l@wehi.edu.au"), 
        person("Shian", "Su", role=c("aut", "cre"), email="su.s@wehi.edu.au"), 
        person("Shalin", "Naik", role=c("ctb"), email="naik.s@wehi.edu.au"), 
        person("Shani", "Amarasinghe", role=c("aut"), email="amarasinghe.s@wehi.edu.au"), 
        person("Oliver", "Voogd", role=c("aut"), email="voogd.o@wehi.edu.au"), 
        person("Phil", "Yang", role=c("aut"), email="yang.p@wehi.edu.au"), 
        person("Matthew", "Ritchie", role=c("ctb"), email="mritchie@wehi.edu.au"))
biocViews: ImmunoOncology, Software, Sequencing, RNASeq,
        GeneExpression, SingleCell, Visualization, SequenceMatching,
        Preprocessing, QualityControl, GenomeAnnotation, DataImport
Description: A preprocessing pipeline for single cell RNA-seq/ATAC-seq data that starts from the fastq files and produces a feature count matrix with associated quality control information. It can process fastq data generated by CEL-seq, MARS-seq, Drop-seq, Chromium 10x and SMART-seq protocols.
Depends: R (>= 4.2.0), SingleCellExperiment
LinkingTo: Rcpp, Rhtslib (>= 1.13.1), testthat
Imports: AnnotationDbi, basilisk, BiocGenerics, biomaRt, Biostrings,
        data.table, dplyr, DropletUtils, flexmix, GenomicRanges,
        GenomicAlignments, GGally, ggplot2, glue (>= 1.3.0), grDevices,
        graphics, hash, IRanges, magrittr, MASS, Matrix (>= 1.5.0),
        mclust, methods, MultiAssayExperiment, org.Hs.eg.db,
        org.Mm.eg.db, purrr, Rcpp (>= 0.11.3), reshape, reticulate,
        Rhtslib, rlang, robustbase, Rsamtools, Rsubread, rtracklayer,
        SummarizedExperiment, S4Vectors, scales, stats, stringr,
        tibble, tidyr, tools, utils, vctrs (>= 0.5.2)
SystemRequirements: C++11, GNU make
License: GPL (>= 2)
Encoding: UTF-8
RoxygenNote: 7.2.3
NeedsCompilation: yes
URL: https://github.com/LuyiTian/scPipe
BugReports: https://github.com/LuyiTian/scPipe
Suggests: BiocStyle, DT, GenomicFeatures, grid, igraph, kableExtra,
        knitr, locStra, plotly, rmarkdown, RColorBrewer, readr,
        reshape2, RANN, shiny, scater (>= 1.11.0), testthat, xml2, umap
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/scPipe
git_branch: RELEASE_3_23
git_last_commit: a390e00
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 23:28:51 UTC; biocbuild
Author: Luyi Tian [aut],
  Shian Su [aut, cre],
  Shalin Naik [ctb],
  Shani Amarasinghe [aut],
  Oliver Voogd [aut],
  Phil Yang [aut],
  Matthew Ritchie [ctb]
Maintainer: Shian Su <su.s@wehi.edu.au>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 05:00:02 UTC; unix
