# Generated by roxygen2: do not edit by hand

export("QC_metrics<-")
export("UMI_dup_info<-")
export("demultiplex_info<-")
export("feature_info<-")
export("feature_type<-")
export("gene_id_type<-")
export(QC_metrics)
export(QC_metrics.sce)
export(TF.IDF.custom)
export(UMI_dup_info)
export(UMI_dup_info.sce)
export(anno_import)
export(anno_to_saf)
export(calculate_QC_metrics)
export(convert_geneid)
export(create_processed_report)
export(create_report)
export(create_sce_by_dir)
export(demultiplex_info)
export(demultiplex_info.sce)
export(detect_outlier)
export(feature_info)
export(feature_info.sce)
export(feature_type)
export(feature_type.sce)
export(gene_id_type)
export(gene_id_type.sce)
export(get_ercc_anno)
export(get_genes_by_GO)
export(get_read_str)
export(organism.sce)
export(plot_QC_pairs)
export(plot_UMI_dup)
export(plot_demultiplex)
export(plot_mapping)
export(remove_outliers)
export(sc_aligning)
export(sc_atac_bam_tagging)
export(sc_atac_cell_calling)
export(sc_atac_create_cell_qc_metrics)
export(sc_atac_create_fragments)
export(sc_atac_create_report)
export(sc_atac_create_sce)
export(sc_atac_emptydrops_cell_calling)
export(sc_atac_feature_counting)
export(sc_atac_filter_cell_calling)
export(sc_atac_peak_calling)
export(sc_atac_pipeline)
export(sc_atac_plot_cells_per_feature)
export(sc_atac_plot_features_per_cell)
export(sc_atac_plot_features_per_cell_ordered)
export(sc_atac_plot_fragments_cells_per_feature)
export(sc_atac_plot_fragments_features_per_cell)
export(sc_atac_plot_fragments_per_cell)
export(sc_atac_plot_fragments_per_feature)
export(sc_atac_remove_duplicates)
export(sc_atac_tfidf)
export(sc_atac_trim_barcode)
export(sc_correct_bam_bc)
export(sc_count_aligned_bam)
export(sc_demultiplex)
export(sc_demultiplex_and_count)
export(sc_detect_bc)
export(sc_exon_mapping)
export(sc_gene_counting)
export(sc_get_umap_data)
export(sc_integrate)
export(sc_interactive_umap_plot)
export(sc_mae_plot_umap)
export(sc_trim_barcode)
exportMethods("QC_metrics<-")
exportMethods("UMI_dup_info<-")
exportMethods("demultiplex_info<-")
exportMethods("feature_info<-")
exportMethods("feature_type<-")
exportMethods("gene_id_type<-")
exportMethods("organism<-")
exportMethods(QC_metrics)
exportMethods(UMI_dup_info)
exportMethods(demultiplex_info)
exportMethods(feature_info)
exportMethods(feature_type)
exportMethods(gene_id_type)
exportMethods(organism)
import(Rcpp)
import(Rhtslib)
import(SingleCellExperiment)
import(org.Hs.eg.db)
import(org.Mm.eg.db)
importClassesFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(AnnotationDbi,mapIds)
importFrom(BiocGenerics,"organism<-")
importFrom(BiocGenerics,organism)
importFrom(GenomicRanges,mcols)
importFrom(Rsamtools,sortBam)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(basilisk,BasiliskEnvironment)
importFrom(basilisk,basiliskRun)
importFrom(basilisk,basiliskStart)
importFrom(basilisk,basiliskStop)
importFrom(biomaRt,getBM)
importFrom(biomaRt,useDataset)
importFrom(biomaRt,useMart)
importFrom(data.table,":=")
importFrom(data.table,copy)
importFrom(data.table,fread)
importFrom(data.table,setkey)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(magrittr,"%>%")
importFrom(mclust,Mclust)
importFrom(mclust,mclustBIC)
importFrom(methods,as)
importFrom(methods,is)
importFrom(reshape,melt)
importFrom(rlang,.data)
importFrom(robustbase,covMcd)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cov)
importFrom(stats,mahalanobis)
importFrom(stats,pchisq)
importFrom(stats,ppoints)
importFrom(stats,prcomp)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,reorder)
importFrom(tibble,column_to_rownames)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,write.table)
useDynLib(scPipe, .registration = TRUE)
